@@ -183,5 +183,31 @@ def validate():
 def detect_joins(target_dataset: str, yes: bool, diff: bool, overwrite: bool):
     from panoramic.cli.command import detect_joins as detect_joins_command
 
-    if not detect_joins_command(target_dataset=target_dataset, diff=diff, overwrite=overwrite, yes=yes):
-        sys.exit(1)
+    detect_joins_command(target_dataset=target_dataset, diff=diff, overwrite=overwrite, yes=yes)
+
+
+@cli.group(name='field')
+def field_cli():
+    """Commands on local field files."""
+
+
+@field_cli.command(help='Clean up fields not linked to any model', cls=ContextAwareCommand)
+@click.option('--target-dataset', '-t', type=str, help='Target a specific dataset')
+@click.option('--yes', '-y', is_flag=True, default=False, help='Automatically confirm all actions')
+@handle_exception
+def cleanup(target_dataset: str, yes: bool):
+    from panoramic.cli.command import delete_orphaned_fields as delete_orphans_command
+
+    delete_orphans_command(target_dataset=target_dataset, yes=yes)
+
+
+@field_cli.command(help='Scaffold fields defined in models', cls=ContextAwareCommand)
+@click.option('--target-dataset', '-t', type=str, help='Target a specific dataset')
+@click.option('--yes', '-y', is_flag=True, default=False, help='Automatically confirm all actions')
+@handle_exception
+def scaffold(target_dataset: str, yes: bool):
+    from panoramic.cli.command import (
+        scaffold_missing_fields as scaffold_missing_fields_command,
+    )
+
+    scaffold_missing_fields_command(target_dataset=target_dataset, yes=yes)
