@@ -5,11 +5,25 @@ from panoramic.cli.file_utils import read_yaml
 from panoramic.cli.paths import FileExtension, PresetFileName, SystemDirectory
 
 
+class GlobalPackage:
+
+    field_files: List[Path]
+
+    def __init__(self, *, field_files: List[Path]) -> None:
+        self.field_files = field_files
+
+    def read_fields(self) -> Iterable[Tuple[Dict[str, Any], Path]]:
+        """Parse field files."""
+        for f in self.field_files:
+            yield read_yaml(f), f
+
+
 class FilePackage:
 
     name: str
     data_source_file: Path
     model_files: List[Path]
+    field_files: List[Path]
 
     def __init__(self, *, name: str, data_source_file: Path, model_files: List[Path], field_files: List[Path]):
         self.name = name
@@ -32,7 +46,14 @@ class FilePackage:
             yield read_yaml(f), f
 
     def __hash__(self) -> int:
-        return hash((self.name, self.data_source_file, self.model_files, self.field_files))
+        return hash(
+            (
+                self.name,
+                self.data_source_file,
+                tuple(self.model_files),
+                tuple(self.field_files),
+            )
+        )
 
     def __eq__(self, o: object) -> bool:
         if not isinstance(o, FilePackage):
@@ -79,6 +100,7 @@ class FileReader:
             for d in package_dirs
         )
 
-    def get_global_fields(self) -> Iterable[Tuple[Dict[str, Any], Path]]:
-        for path in self.cwd.glob(f'{SystemDirectory.FIELDS.value}/*{FileExtension.FIELD_YAML.value}'):
-            yield read_yaml(path), path
+    def get_global_package(self) -> GlobalPackage:
+        return GlobalPackage(
+            field_files=list(self.cwd.glob(f'{SystemDirectory.FIELDS.value}/*{FileExtension.FIELD_YAML.value}'))
+        )
